require('./socket');
require('dotenv').config();
const express = require('express'),
    cors = require('cors'),
    logger = require('morgan'),
    cookieParser = require('cookie-parser'),
    mysql = require('mysql'),
    path = require('path'),
    fs = require('fs');
    
const connection = mysql.createConnection({
    host     : process.env.DB_HOST,
    port     : process.env.DB_PORT,
    user     : process.env.DB_USER,
    password : process.env.DB_PASS,
    database : process.env.DB_DATABASE,
    multipleStatements: true,
});

connection.connect();

/**
 * Get the authorization token from the request header
 * @param {import('express').Request} req
 * @param {import('express').Response} res
 * @param {import('mysql').Connection} sql
 * @param {Function} cb the callback function to call only if the token is valid. There are no parameters.
 */ 
function checkToken(req, res, sql, cb){
    var auth = req.get('Authorization')
    if (!auth){
        renderError(res, 403, 'No authorization token provided');
        return false;
    } else {
        if (auth.includes('Basic ')){
            var token = auth.replace('Basic ', '')
            sql.query("SELECT * FROM Auth WHERE token = ?", token, async (err, result)=>{
                if (err){
                    console.error(err)
                    renderError(res, 500, "Internal Server Error")
                } else {
                    if (result.length < 1){
                        renderError(res, 401, "Authorization Token not found.")
                    } else {
                        if (result[0].validate == 0){
                            renderError(res, 403, "Authorization Token not validated.")
                        } else {
                            console.log('Connexion from '+ result[0].description);
                            cb();
                        }
                    }
                }
            });
        } else {
            renderError(res, 403, "Token not found in Authorization header, please set a Token in Authorization header like this: 'Authorization: Basic [token]'.")
        }
    }
}

/**
 * Render a JSON response for an error
 * @param {import('express').Response} res
 * @param {Number} status
 * @param {String} message
 */
function renderError(res, status, message){
    res.status(status || 500).json({
        error: {
            code: status || 500,
            message: message
        }
    });
}

const port = process.env.PORT || 8080;

const app = express()
    .disable('etag')
    .use(cors())
    .use(express.json())
    .use(cookieParser())
    .use(logger('(:date) :method :url - ":user-agent" (:remote-addr) - :status :response-time ms'));

// User-agent blacklist system (list on user-agent-blacklist.json)
app.use(function(req, res, next){
    var ua = req.get('User-Agent')
    if (!ua){
        renderError(res, 403, "No User-Agent header found, please add a User-Agent header.")
    } else {
        var blacklistUA = JSON.parse(fs.readFileSync(path.join(__dirname, 'user-agent-blacklist.json')))
        var blacklisted = false

        blacklistUA.forEach(bUA=>{
            if (ua.toLowerCase().includes(bUA.toLowerCase())){
                blacklisted = true
            }
        })

        if (blacklisted) {
            console.log('BLACKLISTED UA: ' + ua)
            renderError(res, 403, "Your User-Agent '"+ ua +"' is blacklisted, please change it.")
        } else {
            next()
        }
    }
})

app.get('/', (req, res) => {
    res.json({'online': true})
})

fs.readdirSync(path.join(__dirname, 'route')).filter(file => file.endsWith('.js')).forEach(function(file) {
    require(path.join(__dirname, 'route', file))(app, connection, checkToken, renderError)
});

// catch 404 and forward to error handler
app.use(function(req, res) {
    renderError(res, 404, "Not Found")
});

app.listen(port, () => {
    console.log(`Express server listening on port ${port}`);
});
